import math
from typing import List, Tuple


class Point:
    def __init__(self, x: float, y: float):
        self.x = x
        self.y = y


def to_vec(a: Point, b: Point) -> Point:
    return Point(b.x - a.x, b.y - a.y)


def dist(a: Point, b: Point) -> float:
    return (a.x - b.x) ** 2 + (a.y - b.y) ** 2


def rot1(a: Point) -> Point:
    return Point(a.y, -a.x)


def rot2(a: Point) -> Point:
    return Point(-a.y, a.x)


def cross_product(v1: Point, v2: Point) -> float:
    return v1.x * v2.y - v1.y * v2.x


def vec_mag(v: Point) -> float:
    return math.sqrt(v.x ** 2 + v.y ** 2)


def calcsin(v1: Point, v2: Point) -> float:
    cross = abs(cross_product(v1, v2))
    mag1 = vec_mag(v1)
    mag2 = vec_mag(v2)
    return cross / (mag1 * mag2)


def calccos(v1: Point, v2: Point) -> float:
    return math.sqrt(abs(1 - calcsin(v1, v2) ** 2))


def get_min(a: Point, b: Point, c: Point) -> float:
    if dist(a, c) < dist(a, b):
        return float('inf')
    ac = to_vec(a, c)
    ab = to_vec(a, b)
    if dist(b, c) == dist(a, b) + dist(a, c):
        return float('inf')
    C = math.sqrt(dist(a, c))
    A = abs(calccos(ab, ac) * C)
    B = abs(calcsin(ab, ac) * C)
    return min(A + B, A + C, B + C)

n = int(input())
vec: List[Point] = []

for _ in range(n):
    x, y = map(float, input().split())
    vec.append(Point(x, y))

if n == 2:
    ans = math.sqrt(dist(vec[0], vec[1]))
    print(f"{ans:.10f}")
elif n == 3:
    ans = get_min(vec[0], vec[1], vec[2])
    ans = min(ans, get_min(vec[1], vec[0], vec[2]))
    ans = min(ans, get_min(vec[0], vec[2], vec[1]))
    ans = min(ans, get_min(vec[2], vec[0], vec[1]))
    ans = min(ans, get_min(vec[1], vec[2], vec[0]))
    ans = min(ans, get_min(vec[2], vec[1], vec[0]))
    print(f"{ans:.10f}")
