import math
import sys


class Point:
    def __init__(self, f, s):
        self.f = f
        self.s = s


def to_vec(a, b):
    return Point(b.f - a.f, b.s - a.s)


def dist(a, b):
    return (a.f - b.f) ** 2 + (a.s - b.s) ** 2


def angle(a, b):
    ang = math.atan2(a.s, a.f) - math.atan2(b.s, b.f)
    return ang


def rot1(a):
    return Point(a.s, -a.f)


def rot2(a):
    return Point(-a.s, a.f)


def cross_product(v1, v2):
    return v1.f * v2.s - v1.s * v2.f


def vecmag(v):
    return math.sqrt(v.f * v.f + v.s * v.s)


def calcsin(v1, v2):
    cross = cross_product(v1, v2)
    mag1 = vecmag(v1)
    mag2 = vecmag(v2)
    return cross / (mag1 * mag2)


def calccos(v1, v2):
    return math.sqrt(1 - calcsin(v1, v2) ** 2)


def getmin(a, b, c):
    if dist(a, c) < dist(a, b):
        return 1e9
    ac = to_vec(a, c)
    ab = to_vec(a, b)
    ang = angle(ab, ac)
    if abs(abs(ang) - math.pi / 2.0) < 1e-7:
        return 1e9
    C = math.sqrt(dist(a, c))
    A = abs(calccos(ab, ac) * C)
    B = abs(calcsin(ab, ac) * C)
    return min(A + B, A + C, B + C)



n = int(input())
vec = []
for _ in range(n):
    f, s = map(float, input().split())
    vec.append(Point(f, s))

if n == 2:
    ans = math.sqrt(dist(vec[0], vec[1]))
    print(f"{ans:.10f}")
elif n == 3:
    ans = getmin(vec[0], vec[1], vec[2])
    ans = min(ans, getmin(vec[1], vec[0], vec[2]))
    ans = min(ans, getmin(vec[0], vec[2], vec[1]))
    ans = min(ans, getmin(vec[2], vec[0], vec[1]))
    ans = min(ans, getmin(vec[1], vec[2], vec[0]))
    ans = min(ans, getmin(vec[2], vec[1], vec[0]))
    print(f"{ans:.10f}")